#ifndef SPBLAS_H
#define SPBLAS_H

#ifndef F77NAME
#  define F77NAME(x) x
#endif

#ifdef __cplusplus
extern "C" {
#endif

void scoomm
  (const int transa, const int m, const int n, const int k, 
   const float alpha, 
   const int descra[], const float *val, 
   const int *indx, const int *jndx, const int nnz, 
   const float *b, const int ldb, 
   const float beta, float *c, const int ldc, 
   float *work, const int lwork);

void scscmm
  (const int transa, const int m, const int n, const int k, 
   const float alpha, 
   const int descra[], const float *val, 
   const int *indx, const int *pntr, const float *b, int ldb, 
   const float beta, float *c, const int ldc, 
   float *work, const int lwork);
   
void scsrmm
  (const int transa, const int m, const int n, const int k, 
   const float alpha, 
   const int descra[], const float *val, 
   const int *indx, const int *pntr, const float *b, int ldb, 
   const float beta, float *c, const int ldc, 
   float *work, const int lwork);
   
void dcoomm
  (const int transa, const int m, const int n, const int k, 
   const double alpha, 
   const int descra[], const double *val, 
   const int *indx, const int *jndx, const int nnz, 
   const double *b, const int ldb, 
   const double beta, double *c, const int ldc, 
   double *work, const int lwork);

void dcscmm
  (const int transa, const int m, const int n, const int k, 
   const double alpha, 
   const int descra[], const double *val, 
   const int *indx, const int *pntr, const double *b, int ldb, 
   const double beta, double *c, const int ldc, 
   double *work, const int lwork);
   
void dcsrmm
  (const int transa, const int m, const int n, const int k, 
   const double alpha, 
   const int descra[], const double *val, 
   const int *indx, const int *pntr, const double *b, int ldb, 
   const double beta, double *c, const int ldc, 
   double *work, const int lwork);
   

void dcscsm
  (const int transa, const int m, const int n,
   const int unitd, const double *dv, const double alpha, 
   const int descra[], const double *val, 
   const int *indx, const int *pntr, const double *b, int ldb, 
   const double beta, double *c, const int ldc, 
   double *work, const int lwork);

void dcsrsm
  (const int transa, const int m, const int n,
   const int unitd, const double *dv, const double alpha, 
   const int descra[], const double *val, 
   const int *indx, const int *pntr, const double *b, int ldb, 
   const double beta, double *c, const int ldc, 
   double *work, const int lwork);

void scscsm
  (const int transa, const int m, const int n,
   const int unitd, const float *dv, const float alpha, 
   const int descra[], const float *val, 
   const int *indx, const int *pntr, const float *b, int ldb, 
   const float beta, float *c, const int ldc, 
   float *work, const int lwork);

void scsrsm
  (const int transa, const int m, const int n,
   const int unitd, const float *dv, const float alpha, 
   const int descra[], const float *val, 
   const int *indx, const int *pntr, const float *b, int ldb, 
   const float beta, float *c, const int ldc, 
   float *work, const int lwork);

void dcoosm
  (const int transa, const int m, const int n,
   const int unitd, const double *dv, const double alpha, 
   const int descra[], const double *val, 
   const int *indx, const int *jndx, const int nnz, const double *b, int ldb, 
   const double beta, double *c, const int ldc, 
   double *work, const int lwork);

void scoosm
  (const int transa, const int m, const int n,
   const int unitd, const float *dv, const float alpha, 
   const int descra[], const float *val, 
   const int *indx, const int *jndx, const int nnz, const float *b, int ldb, 
   const float beta, float *c, const int ldc, 
   float *work, const int lwork);

void CoordMatVec_double(int, int, int, const double,
                 const double *, const int *, const int *,
                 const int, const double *, int, const double,
                 double *, int, double *, const int, const int);

void COSymmMatVec_double(int, int, int, const double,
                 const double *, const int *, const int *,
                 const int, const double *, int, const double,
                 double *, int, double *, const int, const int);

void CoordMatVec_float(int, int, int, const float,
                 const float *, const int *, const int *,
                 const int, const float *, int, const float,
                 float *, int, float *, const int, const int);

void COSymmMatVec_float(int, int, int, const float,
                 const float *, const int *, const int *,
                 const int, const float *, int, const float,
                 float *, int, float *, const int, const int);

void CompColMatVec_double(int, int, int, const double,
                   const double *, const int *, const int *,
                   const double *, int, const double, double *, int,
                   double *, const int, const int);

void CCSymmMatVec_double(int, int, int, const double,
                   const double *, const int *, const int *,
                   const double *, int, const double, double *, int,
                   double *, const int, const int);

void CompColMatVec_float(int, int, int, const float,
                   const float *, const int *, const int *,
                   const float *, int, const float, float *, int,
                   float *, const int, const int);

void CCSymmMatVec_float(int, int, int, const float,
                   const float *, const int *, const int *,
                   const float *, int, const float, float *, int,
                   float *, const int, const int);

void CompCol_LU_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompCol_LD_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompCol_UU_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompCol_UD_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompCol_LU_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void CompCol_LD_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void CompCol_UU_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void CompCol_UD_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void CompRowMatVec_double(int, int, int, const double,
                 const double *, const int *, const int *,
                 const double *, int, const double, double *,
                 int, double *, const int, const int);

void CRSymmMatVec_double(int, int, int, const double,
                 const double *, const int *, const int *,
                 const double *, int, const double, double *,
                 int, double *, const int, const int);

void CompRowMatVec_float(int, int, int, const float,
                 const float *, const int *, const int *,
                 const float *, int, const float, float *,
                 int, float *, const int, const int);

void CRSymmMatVec_float(int, int, int, const float,
                 const float *, const int *, const int *,
                 const float *, int, const float, float *,
                 int, float *, const int, const int);


void CompRow_LU_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompRow_LD_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompRow_UU_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompRow_UD_double(int, int, int, const double *,
                 const double, const double *,
                 const int *, const int *, const double *, int,
                 const double, double *, int, double *, int,
                 const int);

void CompRow_LU_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void CompRow_LD_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void CompRow_UU_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void CompRow_UD_float(int, int, int, const float *,
                 const float, const float *,
                 const int *, const int *, const float *, int,
                 const float, float *, int, float *, int,
                 const int);

void ScaleArray_double(int, int, double *, int, const double);
void ScaleArray_float(int, int, float *, int, const float);

#ifdef __cplusplus
}
#endif

#endif

