#include <stdlib.h>                        // System includes
#include <iostream.h>                      // 

#include "compcol_double.h"      // Compressed column matrix header
#include "iohb_double.h"         // Harwell-Boeing matrix I/O header
#include "mvblasd.h"             // MV_Vector level 1 BLAS
#include "diagpre_double.h"      // Diagonal preconditioner

#include "bicg.h"                // IML++ BiCG template

int main(int argc, char * argv[])
{
  if (argc < 2) {
    cerr << "Usage: " << argv[0] << " HBfile " << endl;
    exit(-1);
  }

  double tolerance = 1.e-12;                // Convergence tolerance
  double tol = tolerance;  
  int result1, result2, maxit = 150;       // Maximum iterations

  CompCol_Mat_double A;                    // Create a matrix
  readHB_mat(argv[1], &A);                 // Read matrix data
  VECTOR_double b, x(A.dim(1), 0.0);       // Create rhs, solution vectors
  readHB_rhs(argv[1], &b);                 // Read rhs data

  DiagPreconditioner_double D(A);          // Create diagonal preconditioner

  result1 = BiCG(A, x, b, D, maxit, tol);  // Solve system

  if (result1 == 0 ) {
     cout << "   Convergence after " << maxit << " iterations. " << endl;
  } else {
     cout << "   No convergence after " << maxit << " iterations. "<< endl;
  }

  tol = tolerance;
  maxit = 150;
  x = 0.0;
  CompRow_Mat_double A2(A);

  DiagPreconditioner_double D2(A2);         // Create diagonal preconditioner

  result2 = BiCG(A, x, b, D2, maxit, tol);  // Solve system

  if (result2 == 0 ) {
     cout << "   Convergence after " << maxit << " iterations. " << endl;
  } else {
     cout << "   No convergence after " << maxit << " iterations. "<< endl;
  }

  if ( result1 == 0 && result2 == 0 ) {
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << "   +  Successful completion of BiCG with diagonal preconditioning +" << endl;
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  } else {
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << "   +  BiCG with diagonal preconditioning not successful       +" << endl;
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  }
  return result2;
}
