#include <stdlib.h>                         // System includes
#include <iostream.h>                       // 

#include "compcol_double.h"       // Compressed column matrix header
#include "comprow_double.h"       // Compressed row matrix header
#include "iohb_double.h"          // Harwell-Boeing matrix I/O header
#include "icpre_double.h"         // preconditioner
#include "mvblasd.h"              // MV_Vector level 1 BLAS
#include MATRIX_H                 // MV_Matrix dense matrix header
#include "gmres.h"                // IML++ GMRES template

int main(int argc, char * argv[])
{
  if (argc < 2) {
    cerr << "Usage: " << argv[0] << " HBfile " << endl;
    exit(-1);
  }

  double tolerance = 1.e-12;                // Convergence tolerance
  double tol= tolerance;               
  int result1, result2, maxit = 150, restart = 32;    // Maximum, restart iterations

  CompCol_Mat_double A;                     // Create a matrix
  readHB_mat(argv[1], &A);                  // Read matrix data
  VECTOR_double b, x(A.dim(1), 0.0);        // Create rhs, solution vectors
  readHB_rhs(argv[1], &b);                  // Read rhs data

  MATRIX_double H(restart+1, restart, 0.0); // storage for upper Hessenberg H

  ICPreconditioner_double PRE1(A);    // Create IC preconditioner  
  result1 = GMRES(A, x, b, PRE1, H, restart, maxit, tol); // Solve system
  if (result1 == 0 ) {
     cout << "   Convergence after " << maxit << " iterations. " << endl;
  } else {
     cout << "   No convergence after " << maxit << " iterations. "<< endl;
  }

  CompRow_Mat_double R(A);
  ICPreconditioner_double PRE2(R);    // Create IC preconditioner  

  H  = 0.0;
  x = 0.0;
  tol = tolerance;                       // Convergence tolerance
  maxit = 150; 
  restart = 32;                      // Maximum, restart iterations
  result2 = GMRES(A, x, b, PRE2, H, restart, maxit, tol); // Solve system
  if (result2 == 0 ) {
     cout << "   Convergence after " << maxit << " iterations.  " << endl;
  } else {
     cout << "   No convergence after " << maxit << " iterations. " << endl;
  }

  if ( result1 == 0 && result2 == 0 ) {
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << "   +  Successful completion of GMRES with IC  preconditioning +" << endl;
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  } else {
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << "   +  GMRES with IC  preconditioning not successful           +" << endl;
  cout << "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  }
  return result2;
}
